#include "nthreads.h"

unsigned RecommendedThreads(const unsigned desiredthreads)
{
 errno=0;
 long retsys=sysconf(_SC_NPROCESSORS_ONLN);
 if (retsys==-1)
 {
  Rcpp::Rcout << "Warning: the system call 'sysconf(_SC_NPROCESSORS_ONLN)' in this system does not return the number of cores.\n";
  if (errno==EINVAL)
   Rcpp::Rcout << "The error is '_SC_NPROCESSORS_ONLN is an invalid argument`. Probably your system is not POSIX.2 compliant.\n";
  else
   Rcpp::Rcout << "The error number returned by the system call is " << errno << " (consult errno.h or 'man errno' for information).\n";
  Rcpp::Rcout << "This function will return " << desiredthreads << ", as you have requested, but this may not improve performance.\n";
  Rcpp::Rcout << "(It might indeed degrade it, if too many threads are requested).\n";
  return(desiredthreads);
 }

 if (retsys==0)
 {
  Rcpp::Rcout << "Warning: unknown error in system call to  'sysconf(_SC_NPROCESSORS_ONLN)'. It returns 0.\n";
  Rcpp::Rcout << "The NumCores function will return " << desiredthreads << ", as you have requested, but this may not improve performance.\n";
  Rcpp::Rcout << "(It might indeed degrade it, if too many threads are requested).\n";
  return(desiredthreads);
 }

 unsigned available_cores=unsigned(retsys);

 if (desiredthreads<=0)
 {
  Rcpp::Rcout << "This system has " << available_cores << " available cores, so " << available_cores << " threads will be created if you pass -1 to the PValuesEstimation function.\n";
  return(available_cores);
 }

 if (desiredthreads<=available_cores)
 {
  Rcpp::Rcout << "This system has " << available_cores << " available cores, so you can request for " << desiredthreads << " threads if you wish.\n";
  return(desiredthreads);
 }
  
 if (desiredthreads>available_cores)
 {
  Rcpp::Rcerr << "Warning: this sytem has " << available_cores << " available cores. Using " << desiredthreads << " threads might degrade performance.\n";
  return(desiredthreads);
 }

 // Just to avoid a compiler warning
 return desiredthreads;
}

