#ifndef PTCOMPUTING_BLOCK_H
#define PTCOMPUTING_BLOCK_H

#include<vector>
#include<string>
#include<fstream>

#include "outinfo.h"

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
  
// Gaussian and Student's t distribution for calculation of p-value from the t-Statistic
#include <boost/math/distributions/normal.hpp>
#include <boost/math/distributions/students_t.hpp>

// Incomplete beta functions used to calculate the integrals in the delta method
#include <boost/math/special_functions/beta.hpp>

#include "betaestimator.h"

// Constants to define the possibilities of the distributions to use for p-value estimation
// If we choose a distribution, p-value is estimated with both, the Montecarlo and the new Beta-based method
// Anyway, Montecarlo estimation doesn't increase substantially the computation time...

#define USE_NORMAL                1
#define USE_TSTUDENT              2
#define USE_BOTH                  3

// Constants to define the possible estimation methods for the p-values
#define NUM_P_ESTIM_METHODS 4
#define P_ESTIM_MONTECARLO_NORMAL 0
#define P_ESTIM_MONTECARLO_TSTUD  1
#define P_ESTIM_BETA_NORMAL       2
#define P_ESTIM_BETA_TSTUD        3

// Macros to improve readability when an array of values is returned (filled inside a function)
#define MEAN1(m) m[0]
#define MEAN2(m) m[2]
#define VAR1(m)  m[1]
#define VAR2(m)  m[3]

class PTComputingBlock
{
 public:
	PTComputingBlock(	Rcpp::NumericMatrix 	& tMatrix,
			unsigned			DegreesOfFreedom,
			unsigned 		 	EstimationMethods,
			REAL_TYPE	 		vlim_cf,
			unsigned			InitialGene,
			unsigned			FinalGene,
			Rcpp::NumericMatrix		& P,
			Rcpp::NumericMatrix		& PConfBetaNor,
			Rcpp::NumericMatrix		& PConfBetaTst,
			std::string			debug_file
		);
	void RunComputation();
 private:
	Rcpp::NumericMatrix *T;
	unsigned df;
	unsigned dts; 
	REAL_TYPE lim_cf;
	unsigned startgene;
	unsigned endgene;

	unsigned nsim;

	const unsigned approx_order=16;

	Rcpp::NumericMatrix *Pres;
	Rcpp::NumericMatrix *PConfBetaNorRes;
	Rcpp::NumericMatrix *PConfBetaTstRes;

	int outdebug;
	std::ofstream debfile;

        REAL_TYPE DeltaMethod(uint32_t sample_size,REAL_TYPE p_central,REAL_TYPE s_alpha,REAL_TYPE s_beta,unsigned gene);

	void EstimatePValuesAndConfidence(const std::vector<REAL_TYPE> & Tg,REAL_TYPE t0,unsigned df,unsigned dist_to_use,
					  REAL_TYPE pvalues[NUM_P_ESTIM_METHODS],bool get_interval, REAL_TYPE Nor[2],REAL_TYPE Tst[2],unsigned gene);
};

#endif

