#ifndef _PVALEST_H
#define _PVALEST_H

// These are for the call to sysconf and errno
#include <unistd.h>
#include <errno.h>
// This is obviously for calling std::thread
#include <thread>
// This is to process one of teh parameters, which is a string
#include <string>

#include  <Rcpp.h>

// The class computingblock is needed in the PValuesEstimation function
#include "computingblock.h"

//' Integrated p-values
//'
//' This function accepts the raw p-value and the p-values corresponding to each of the random labellings, and finds the
//' Montecarlo p-value and the integrated p-value using either a normal distribution, a Student's-t distribution, or both.
//' @param data Expression matrix. It must be passed as a matrix which is the transpose of the usual expression matrix, i.e. with cases as rows and genes/probes as columns.
//' @param alpha Level of confidence used for calculation of the confidence interval of the integrated p-value with the Delta method. Use a negative value to indicate that confidence intervals must not be calculated.
//' @param combinations Matrix of random labellings. It is a matrix of booleans where each random labelling is a row and each column has the boolean value 'TRUE' for signaling the individual is a control or 'FALSE' to mark is as a case. Each row must have the same number of 'TRUE' values. This matrix can be generated with the ManyCombinations function.
//' @param cases A row vector of booleans indicating which individual is a case (with 'TRUE') and which one is a control (with 'FALSE').
//' @param dts String to choose the distribution used to calculate the integrated p-values. The only accepted values are "normal", "tstudent" or "both".
//' @param numthreads The number of computation threads to use. Possible values are -1 for NOT using threads at all, 0 for choosing automatically the optimal number of threads and any possitive integer to request such number of threads.
//' @param deb_filename String with the name of the file or files where numerical errors are emitted. It can be: the empty string, "", if you don't want to see the errors; "Rcerr" to send the errors to the usual Rcpp error output; any other string like "a" to send the errors to that file or, if several threads are used, to files "a000", "a001", ... 
//' @export 
// [[Rcpp::export]]
Rcpp::DataFrame PValuesEstimation(Rcpp::NumericMatrix data,double alpha,Rcpp::LogicalMatrix combinations,Rcpp::LogicalMatrix cases,Rcpp::String dts,int numthreads,Rcpp::String deb_filename);

#endif

