#ifndef _PVALFTV_H
#define _PVALFTV_H

// These are for the call to sysconf and errno
#include <unistd.h>
#include <errno.h>
// This is obviously for calling std::thread
#include <thread>
// This is to process one of the parameters, which is a string
#include <string>

#include  <Rcpp.h>

// The class computingblocktv is needed in the PValuesFromTValues function
#include "ptcomputingblock.h"

//' Integrated p-values from statistic
//'
//' This function accepts the statistic corresponding to the original value
//' and to each of the random rearrangements, and finds the
//' Montecarlo p-value and the integrated p-value using either a normal
//' distribution, a Student's-t distribution, or both.
//' @param t Statistic matrix. It must be passed as a matrix with the
//' statistic value of the original data as its first row and the
//' corresponding value for all rearrangements as the other rows
//' @param df Number of degrees of freedom if a t-student distribution is
//' used (see parameter dts). If t-student is not requested this parameter
//' is ignored.
//' @param alpha Level of confidence used for calculation of the confidence
//' interval of the integrated p-value with the Delta method. Use a negative
//' value to indicate that confidence intervals must not be calculated.
//' @param dts String to choose the distribution used to calculate the
//' integrated p-values. The only accepted values are "normal", "tstudent"
//' or "both".
//' @param numthreads The number of computation threads to use.
//' Possible values are -1 for NOT using threads at all, 0 for choosing
//' automatically the optimal number of threads and any possitive integer
//' to request such number of threads.
//' @param deb_filename String with the name of the file or files where numerical errors are emitted. It can be: the empty string, "", if you don't want to see the errors; "Rcerr" to send the errors to the usual Rcpp error output; any other string like "a" to send the errors to that file or, if several threads are used, to files "a000", "a001", ... 
//' @export 
// [[Rcpp::export]]
Rcpp::DataFrame PValuesFromStatistic(Rcpp::NumericMatrix t,unsigned int df,double alpha,Rcpp::String dts,int numthreads,Rcpp::String deb_filename);

#endif

